@@ -1,26 +1,19 @@
-try:
-    from collections import OrderedDict
-except ImportError:
-    # Python 2.6
-    from ordereddict import OrderedDict
 import json
-import os
-import sys
 
 import numpy as np
-from pandas import DataFrame, Series, Index
-from shapely.geometry import mapping, shape
+import pandas as pd
+from pandas import DataFrame, Series
+from shapely.geometry import mapping, shape, Point
 from shapely.geometry.base import BaseGeometry
-from six import string_types
+from six import string_types, PY3
 
-from geopandas import GeoSeries
-from geopandas.base import GeoPandasBase
+from geopandas.base import GeoPandasBase, _CoordinateIndexer
+from geopandas.geoseries import GeoSeries
 from geopandas.plotting import plot_dataframe
 import geopandas.io
 
 
 DEFAULT_GEO_COLUMN_NAME = 'geometry'
-PY3 = sys.version_info[0] == 3
 
 
 class GeoDataFrame(GeoPandasBase, DataFrame):
@@ -29,8 +22,8 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
     with geometry. In addition to the standard DataFrame constructor arguments,
     GeoDataFrame also accepts the following keyword arguments:
 
-    Keyword Arguments
-    -----------------
+    Parameters
+    ----------
     crs : str (optional)
         Coordinate system
     geometry : str or array (optional)
@@ -96,7 +89,7 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
         Parameters
         ----------
-        keys : column label or array
+        col : column label or array
         drop : boolean, default True
             Delete column to be used as the new geometry
         inplace : boolean, default False
@@ -152,7 +145,7 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
             level.crs = crs
 
         # Check that we are using a listlike of geometries
-        if not all(isinstance(item, BaseGeometry) or not item for item in level):
+        if not all(isinstance(item, BaseGeometry) or pd.isnull(item) for item in level):
             raise TypeError("Input geometry column must contain valid geometry objects.")
         frame[geo_column_name] = level
         frame._geometry_column_name = geo_column_name
@@ -163,28 +156,75 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
     @classmethod
     def from_file(cls, filename, **kwargs):
-        """
-        Alternate constructor to create a GeoDataFrame from a file.
+        """Alternate constructor to create a ``GeoDataFrame`` from a file.
 
-        Example:
-            df = geopandas.GeoDataFrame.from_file('nybb.shp')
+        Can load a ``GeoDataFrame`` from a file in any format recognized by
+        `fiona`. See http://fiona.readthedocs.io/en/latest/manual.html for details.
 
-        Wraps geopandas.read_file(). For additional help, see read_file()
+        Parameters
+        ----------
 
+        filename : str
+            File path or file handle to read from. Depending on which kwargs
+            are included, the content of filename may vary. See
+            http://fiona.readthedocs.io/en/latest/README.html#usage for usage details.
+        kwargs : key-word arguments
+            These arguments are passed to fiona.open, and can be used to
+            access multi-layer data, data stored within archives (zip files),
+            etc.
+
+        Examples
+        --------
+
+        >>> df = geopandas.GeoDataFrame.from_file('nybb.shp')
         """
         return geopandas.io.file.read_file(filename, **kwargs)
 
     @classmethod
-    def from_features(cls, features, crs=None):
+    def from_features(cls, features, crs=None, columns=None):
         """
         Alternate constructor to create GeoDataFrame from an iterable of
-        features. Each element must be a feature dictionary or implement
-        the __geo_interface__.
-        See: https://gist.github.com/sgillies/2217756
+        features or a feature collection.
+
+        Parameters
+        ----------
+        features
+            - Iterable of features, where each element must be a feature
+              dictionary or implement the __geo_interface__.
+            - Feature collection, where the 'features' key contains an
+              iterable of features.
+            - Object holding a feature collection that implements the
+              ``__geo_interface__``.
+        crs : str or dict (optional)
+            Coordinate reference system to set on the resulting frame.
+        columns : list of column names, optional
+            Optionally specify the column names to include in the output frame.
+            This does not overwrite the property names of the input, but can
+            ensure a consistent output format.
+
+        Returns
+        -------
+        GeoDataFrame
+
+        Notes
+        -----
+        For more information about the ``__geo_interface__``, see
+        https://gist.github.com/sgillies/2217756
 
         """
+        # Handle feature collections
+        if hasattr(features, "__geo_interface__"):
+            fs = features.__geo_interface__
+        else:
+            fs = features
+
+        if isinstance(fs, dict) and fs.get('type') == 'FeatureCollection':
+            features_lst = fs['features']
+        else:
+            features_lst = features
+
         rows = []
-        for f in features:
+        for f in features_lst:
             if hasattr(f, "__geo_interface__"):
                 f = f.__geo_interface__
             else:
@@ -193,57 +233,93 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
             d = {'geometry': shape(f['geometry']) if f['geometry'] else None}
             d.update(f['properties'])
             rows.append(d)
-        df = GeoDataFrame.from_dict(rows)
+        df = GeoDataFrame(rows, columns=columns)
         df.crs = crs
         return df
 
     @classmethod
-    def from_postgis(cls, sql, con, geom_col='geom', crs=None, index_col=None,
-                     coerce_float=True, params=None):
+    def from_postgis(cls, sql, con, geom_col='geom', crs=None,
+                     index_col=None, coerce_float=True,
+                     parse_dates=None, params=None):
         """
-        Alternate constructor to create a GeoDataFrame from a sql query
-        containing a geometry column.
+        Alternate constructor to create a ``GeoDataFrame`` from a sql query
+        containing a geometry column in WKB representation.
 
-        Example:
-            df = geopandas.GeoDataFrame.from_postgis(con,
-                "SELECT geom, highway FROM roads;")
-
-        Wraps geopandas.read_postgis(). For additional help, see read_postgis()
+        Parameters
+        ----------
+        sql : string
+        con : DB connection object or SQLAlchemy engine
+        geom_col : string, default 'geom'
+            column name to convert to shapely geometries
+        crs : optional
+            Coordinate reference system to use for the returned GeoDataFrame
+        index_col : string or list of strings, optional, default: None
+            Column(s) to set as index(MultiIndex)
+        coerce_float : boolean, default True
+            Attempt to convert values of non-string, non-numeric objects (like
+            decimal.Decimal) to floating point, useful for SQL result sets
+        parse_dates : list or dict, default None
+            - List of column names to parse as dates.
+            - Dict of ``{column_name: format string}`` where format string is
+              strftime compatible in case of parsing string times, or is one of
+              (D, s, ns, ms, us) in case of parsing integer timestamps.
+            - Dict of ``{column_name: arg dict}``, where the arg dict
+              corresponds to the keyword arguments of
+              :func:`pandas.to_datetime`. Especially useful with databases
+              without native Datetime support, such as SQLite.
+        params : list, tuple or dict, optional, default None
+            List of parameters to pass to execute method.
 
+        Examples
+        --------
+        >>> sql = "SELECT geom, highway FROM roads"
+        SpatiaLite
+        >>> sql = "SELECT ST_Binary(geom) AS geom, highway FROM roads"
+        >>> df = geopandas.GeoDataFrame.from_postgis(sql, con)
         """
-        return geopandas.io.sql.read_postgis(sql, con, geom_col, crs, index_col,
-                     coerce_float, params)
+
+        df = geopandas.io.sql.read_postgis(
+                sql, con, geom_col=geom_col, crs=crs,
+                index_col=index_col, coerce_float=coerce_float,
+                parse_dates=parse_dates, params=params)
+
+        return df
 
     def to_json(self, na='null', show_bbox=False, **kwargs):
         """
-        Returns a GeoJSON string representation of the GeoDataFrame.
+        Returns a GeoJSON representation of the ``GeoDataFrame`` as a string.
 
         Parameters
         ----------
         na : {'null', 'drop', 'keep'}, default 'null'
-            Indicates how to output missing (NaN) values in the GeoDataFrame
-            * null: output the missing entries as JSON null
-            * drop: remove the property from the feature. This applies to
-                    each feature individually so that features may have
-                    different properties
-            * keep: output the missing entries as NaN
-
-        show_bbox : include bbox (bounds) in the geojson
+            Indicates how to output missing (NaN) values in the GeoDataFrame.
+            See below.
+        show_bbox : bool, optional, default: False
+            Include bbox (bounds) in the geojson
 
+        Notes
+        -----
         The remaining *kwargs* are passed to json.dumps().
 
+        Missing (NaN) values in the GeoDataFrame can be represented as follows:
+
+        - ``null``: output the missing entries as JSON null.
+        - ``drop``: remove the property from the feature. This applies to each
+          feature individually so that features may have different properties.
+        - ``keep``: output the missing entries as NaN.
         """
         return json.dumps(self._to_geo(na=na, show_bbox=show_bbox), **kwargs)
 
     @property
     def __geo_interface__(self):
-        """
-        Returns a python feature collection (i.e. the geointerface)
-        representation of the GeoDataFrame.
+        """Returns a ``GeoDataFrame`` as a python feature collection.
+
+        Implements the `geo_interface`. The returned python data structure
+        represents the ``GeoDataFrame`` as a GeoJSON-like
+        ``FeatureCollection``.
 
         This differs from `_to_geo()` only in that it is a property with
         default args instead of a method
-
         """
         return self._to_geo(na='null', show_bbox=True)
 
@@ -263,45 +339,50 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
             * keep: output the missing entries as NaN
 
         show_bbox : include bbox (bounds) in the geojson. default False
-
         """
-        def fill_none(row):
-            """
-            Takes in a Series, converts to a dictionary with null values
-            set to None
-
-            """
-            na_keys = row.index[row.isnull()]
-            d = row.to_dict()
-            for k in na_keys:
-                d[k] = None
-            return d
-
-        # na_methods must take in a Series and return dict
-        na_methods = {'null': fill_none,
-                      'drop': lambda row: row.dropna().to_dict(),
-                      'keep': lambda row: row.to_dict()}
-
-        if na not in na_methods:
+        if na not in ['null', 'drop', 'keep']:
             raise ValueError('Unknown na method {0}'.format(na))
-        f = na_methods[na]
 
-        for i, row in self.iterrows():
-            properties = f(row)
-            del properties[self._geometry_column_name]
+        ids = np.array(self.index, copy=False)
+        geometries = np.array(self[self._geometry_column_name], copy=False)
 
-            feature = {
-                'id': str(i),
-                'type': 'Feature',
-                'properties': properties,
-                'geometry': mapping(row[self._geometry_column_name])
-                            if row[self._geometry_column_name] else None
-            }
+        properties_cols = self.columns.difference([self._geometry_column_name])
 
-            if show_bbox:
-                feature['bbox'] = row.geometry.bounds
+        if len(properties_cols) > 0:
+            # convert to object to get python scalars.
+            properties = self[properties_cols].astype(object).values
+            if na == 'null':
+                properties[pd.isnull(self[properties_cols]).values] = None
 
-            yield feature
+            for i, row in enumerate(properties):
+                geom = geometries[i]
+
+                if na == 'drop':
+                    properties_items = dict((k, v) for k, v
+                                            in zip(properties_cols, row)
+                                            if not pd.isnull(v))
+                else:
+                    properties_items = dict((k, v) for k, v
+                                            in zip(properties_cols, row))
+
+                feature = {'id': str(ids[i]),
+                           'type': 'Feature',
+                           'properties': properties_items,
+                           'geometry': mapping(geom) if geom else None}
+
+                if show_bbox:
+                    feature['bbox'] = geom.bounds if geom else None
+                yield feature
+
+        else:
+            for fid, geom in zip(ids, geometries):
+                feature = {'id': str(fid),
+                           'type': 'Feature',
+                           'properties': {},
+                           'geometry': mapping(geom) if geom else None}
+                if show_bbox:
+                        feature['bbox'] = geom.bounds if geom else None
+                yield feature
 
     def _to_geo(self, **kwargs):
         """
@@ -313,16 +394,18 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
                'features': list(self.iterfeatures(**kwargs))}
 
         if kwargs.get('show_bbox', False):
-            geo['bbox'] = self.total_bounds
+            geo['bbox'] = tuple(self.total_bounds)
 
         return geo
 
     def to_file(self, filename, driver="ESRI Shapefile", schema=None,
                 **kwargs):
-        """
-        Write this GeoDataFrame to an OGR data source
+        """Write the ``GeoDataFrame`` to a file.
+
+        By default, an ESRI shapefile is written, but any OGR data source
+        supported by Fiona can be written. A dictionary of supported OGR
+        providers is available via:
 
-        A dictionary of supported OGR providers is available via:
         >>> import fiona
         >>> fiona.supported_drivers
 
@@ -330,32 +413,44 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         ----------
         filename : string
             File path or file handle to write to.
-        driver : string, default 'ESRI Shapefile'
+        driver : string, default: 'ESRI Shapefile'
             The OGR format driver used to write the vector file.
-        schema : dict, default None
+        schema : dict, default: None
             If specified, the schema dictionary is passed to Fiona to
             better control how the file is written.
 
-        The *kwargs* are passed to fiona.open and can be used to write
-        to multi-layer data, store data within archives (zip files), etc.
+        Notes
+        -----
+        The extra keyword arguments ``**kwargs`` are passed to fiona.open and
+        can be used to write to multi-layer data, store data within archives
+        (zip files), etc.
         """
         from geopandas.io.file import to_file
         to_file(self, filename, driver, schema, **kwargs)
 
     def to_crs(self, crs=None, epsg=None, inplace=False):
-        """Transform geometries to a new coordinate reference system
+        """Transform geometries to a new coordinate reference system.
 
-        This method will transform all points in all objects.  It has
-        no notion or projecting entire geometries.  All segments
-        joining points are assumed to be lines in the current
-        projection, not geodesics.  Objects crossing the dateline (or
-        other projection boundary) will have undesirable behavior.
+        Transform all geometries in a GeoSeries to a different coordinate
+        reference system.  The ``crs`` attribute on the current GeoSeries must
+        be set.  Either ``crs`` in string or dictionary form or an EPSG code
+        may be specified for output.
 
-        `to_crs` passes the `crs` argument to the `Proj` function from the
-        `pyproj` library (with the option `preserve_units=True`). It can
-        therefore accept proj4 projections in any format
-        supported by `Proj`, including dictionaries, or proj4 strings.
+        This method will transform all points in all objects.  It has no notion
+        or projecting entire geometries.  All segments joining points are
+        assumed to be lines in the current projection, not geodesics.  Objects
+        crossing the dateline (or other projection boundary) will have
+        undesirable behavior.
 
+        Parameters
+        ----------
+        crs : dict or str
+            Output projection parameters as string or in dictionary form.
+        epsg : int
+            EPSG code specifying output projection.
+        inplace : bool, optional, default: False
+            Whether to return a new GeoDataFrame or do the transformation in
+            place.
         """
         if inplace:
             df = self
@@ -443,7 +538,15 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         return GeoDataFrame(data).__finalize__(self)
 
     def plot(self, *args, **kwargs):
+        """Generate a plot of the geometries in the ``GeoDataFrame``.
 
+        If the ``column`` parameter is given, colors plot according to values
+        in that column, otherwise calls ``GeoSeries.plot()`` on the
+        ``geometry`` column.
+
+        Wraps the ``plot_dataframe()`` function, and documentation is copied
+        from there.
+        """
         return plot_dataframe(self, *args, **kwargs)
 
     plot.__doc__ = plot_dataframe.__doc__
@@ -486,7 +589,7 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
         g = self.groupby(by=by, group_keys=False)[self.geometry.name].agg(merge_geometries)
 
         # Aggregate
-        aggregated_geometry = GeoDataFrame(g, geometry=self.geometry.name)
+        aggregated_geometry = GeoDataFrame(g, geometry=self.geometry.name, crs=self.crs)
         # Recombine
         aggregated = aggregated_geometry.join(aggregated_data)
 
@@ -496,6 +599,73 @@ class GeoDataFrame(GeoPandasBase, DataFrame):
 
         return aggregated
 
+    # overrides GeoPandasBase method
+    def explode(self):
+        """
+        Explode muti-part geometries into multiple single geometries.
+
+        Each row containing a multi-part geometry will be split into
+        multiple rows with single geometries, thereby increasing the vertical
+        size of the GeoDataFrame.
+
+        The index of the input geodataframe is no longer unique and is
+        replaced with a multi-index (original index with additional level
+        indicating the multiple geometries: a new zero-based index for each
+        single part geometry per multi-part geometry).
+
+        Returns
+        -------
+        GeoDataFrame
+            Exploded geodataframe with each single geometry
+            as a separate entry in the geodataframe.
+
+        """
+        df_copy = self.copy()
+
+        exploded_geom = df_copy.geometry.explode().reset_index(level=-1)
+        exploded_index = exploded_geom.columns[0]
+
+        df = pd.concat(
+            [df_copy.drop(df_copy._geometry_column_name, axis=1),
+             exploded_geom], axis=1)
+        # reset to MultiIndex, otherwise df index is only first level of
+        # exploded GeoSeries index.
+        df.set_index(exploded_index, append=True, inplace=True)
+        df.index.names = list(self.index.names) + [None]
+        geo_df = df.set_geometry(self._geometry_column_name)
+        return geo_df
+
+
+def points_from_xy(x, y, z=None):
+    """
+    Generate list of shapely Point geometries from x, y(, z) coordinates.
+
+    Parameters
+    ----------
+    x, y, z : array
+
+    Examples
+    --------
+    >>> geometry = geopandas.points_from_xy(x=[1, 0], y=[0, 1])
+    >>> geometry = geopandas.points_from_xy(df['x'], df['y'], df['z'])
+    >>> gdf = geopandas.GeoDataFrame(
+            df, geometry=geopandas.points_from_xy(df['x'], df['y']))
+
+    Returns
+    -------
+    list : list
+    """
+    if not len(x) == len(y):
+        raise ValueError("x and y arrays must be equal length.")
+    if z is not None:
+        if not len(z) == len(x):
+            raise ValueError("z array must be same length as x and y.")
+        geom = [Point(i, j, k) for i, j, k in zip(x, y, z)]
+    else:
+        geom = [Point(i, j) for i, j in zip(x, y)]
+    return geom
+
+
 def _dataframe_set_geometry(self, col, drop=False, inplace=False, crs=None):
     if inplace:
         raise ValueError("Can't do inplace setting when converting from"
@@ -510,3 +680,6 @@ else:
     import types
     DataFrame.set_geometry = types.MethodType(_dataframe_set_geometry, None,
                                               DataFrame)
+
+
+GeoDataFrame._create_indexer('cx', _CoordinateIndexer)
